c  ====================================================================
      real function as(t,x,y,iq,jq,ider)
c     .. Real/..
      implicit real (a-h,o-z)
c     .. Parameters ..
      parameter(mm=51,mq=2)
c     .. Common block for equation parameters ..
      common/eqn/mt,mx,my,neqn,peqn(100),
     >  acoef(0:mm,0:mm,0:mm,mq,mq,6),
     >  tmpt(0:mm),tmpx(0:mm),tmpy(0:mm)
c     .. User-supplied subroutine ..
      external ran1
c     .. Save between calls ..
      save tpi
c     .. 
c
c  Purpose
c  =======
c
c  Function as evaluates the variable coefficients of a random elliptic
c  operator L at the point (t,x,y).
c
c  Method
c  ======
c  
c  On the first call to as, ider should be set to zero, and random
c  Fourier coefficients are then evaluated for each variable desired.
c  These Fourier coefficients are stored in the common block 'eqn'
c  in the array acoef. The rest of the common block is devoted to
c  temporary arrays tmpt, tmpx and tmpy for the evaluation phase.
c  On later calls, the desired coefficient is evaluated at (t,x,y).
c
c  Ellipticity and invertibility of the operator are enforced as follows:
c  First, 6*nq*nq random Fourier series are generated.
c  Then ellipticity (uniform positive definiteness of the principal part)
c  is enforced by setting the principal coefficients of the diagonal i=j to
c  
c     a^ii_xx= 1 + f^ii_1^2 + f^ii_2^2
c
c     a^ii_xy= 2 * f^ii_1 * f^ii_2
c
c     a^ii_yy= 1 + f^ii_2^2
c
c  and invertibility is enforced by setting the zero-order coefficient
c  of the diagonal iq=jq to
c
c     c= - f^ii_6^2.
c
c  The constant beta=peqn(1) allows the user to influence the strength
c  of the convection coefficients, defined by
c
c     bx= beta * f4
c
c     by= beta * f5.
c
c  The off-diagonal entries are just the random Fourier series.
c
c  The operator is defined by
c
c     (Lu)_i  = \sum_ j=1,nq a^ij_xx u^j_xx + a^ij_xy u^j_xy + a^ij_yy u^j_yy +
c        b^ij_x u^j_x + b^ij_y u^j_y + c^ij u^j.
c
c  Arguments 
c  =========
c
c  t 	- real.
c         Time variable: unchanged on exit.
c
c  x 	- real.
c         First space variable: unchanged on exit.
c
c  y 	- real.
c         Second space variable: unchanged on exit.
c
c  ider - integer.
c         Derivative selector: ider should be between 1 and 6 inclusive,
c         and indicates the following coefficients:
c         ider=1  axx
c              2  axy
c              3  ayy
c              4  bx
c              5  by
c              6  c 
c         
c  Other Routines Called 
c  =====================
c
c  ran1
c  arrran1
c
c  ====================================================================
c
c  .. Executable Statements ..
c
      if(ider.le.0)then
c        Initialize the random Fourier coefficients on [-1,1].
	 ml=mm+1
c        Initialize random number generator by setting idum=-1.
c        This ensures that we solve the same problem for the same
c        choice of beta.
         ffggbb=ran1(idum)
c        If mt is increased from one run to the next, we only 
c        use more from the same set of Fourier coefficients,
c        we don't change the ones already in use.
	 idum=-1
	 do 10 i=1,mq
	 do 10 j=1,mq
	 do 10 idr=1,6
	 do 10 ix=0,mm
	 do 10 iy=0,mm
	    call arrran1(idum,ml,-1.0,1.0,acoef(0,ix,iy,i,j,idr))
10       continue
         tpi=2.0*3.1415926535898
	 return
      end if

c     Now we evaluate the matrix elements.
      pt=tpi*t
      px=tpi*x
      py=tpi*y
c     Evaluate cosines and store in tmp arrays.
      tmpt(0)=1.0
      do 20 it=1,mt
         tmpt(it)=cos(pt*it)
20    continue
      tmpx(0)=1.0
      do 30 ix=1,mx
         tmpx(ix)=cos(px*ix)
30    continue
      tmpy(0)=1.0
      do 40 iy=1,my
         tmpy(iy)=cos(py*iy)
40    continue

c     Put elliptic operators on main diagonal.
      if(iq.eq.jq)then
      if(ider.le.3)then
c        Need all three matrix elements to get each coeff.
c        First evaluate triangular matrix, then square and add
c        identity to get positive definite matrix A > I.
         axx=0.0
         axy=0.0
         ayy=0.0
	 as=0.0
         do 100 it=0,mt
            ct=tmpt(it)
         do 100 ix=0,mx
            cx=ct*tmpx(ix)
         do 100 iy=0,my
            cc=cx*tmpy(iy)
            axx=axx+acoef(it,ix,iy,iq,jq,1)*cc
            axy=axy+acoef(it,ix,iy,iq,jq,2)*cc
            ayy=ayy+acoef(it,ix,iy,iq,jq,3)*cc
100      continue
         if(ider.eq.1) as=1.0+axx**2+axy**2
         if(ider.eq.2) as=2.0*axy*ayy
         if(ider.eq.3) as=1.0+ayy**2
c        Done evaluating the desired element.
	 return
      end if

      if(ider.le.5)then
c        First-order terms are arbitrary random cosine series,
c        multiplied by convection coefficient beta.
	 as=0.0
         do 200 it=0,mt
            ct=tmpt(it)
         do 200 ix=0,mx
            cx=ct*tmpx(ix)
         do 200 iy=0,my
            cc=cx*tmpy(iy)
            as=as+acoef(it,ix,iy,iq,jq,ider)*cc
200      continue
         as=peqn(1)*as
c        Done evaluating the desired element.
         return
      end if

c     Zero-order term is negative.
      as=0.0
      do 300 it=0,mt
         ct=tmpt(it)
      do 300 ix=0,mx
         cx=ct*tmpx(ix)
      do 300 iy=0,my
         cc=cx*tmpy(iy)
         as=as+acoef(it,ix,iy,iq,jq,6)*cc
300   continue
c     Negative square guarantees negativity.
      as=-as**2

      else
c     Off diagonal, first and zero order terms only.
      as=0.0
      if(ider.le.3)return
      
c     Sum random Fourier series.
      as=0.0
      do 400 it=0,mt
         ct=tmpt(it)
      do 400 ix=0,mx
         cx=ct*tmpx(ix)
      do 400 iy=0,my
         cc=cx*tmpy(iy)
         as=as+acoef(it,ix,iy,iq,jq,6)*cc
400   continue

      return
      end if
     
      return
      end
