      subroutine eqn_cfp(n,ud,a,b,t,as,fv,df,nq,nd,mapcf,mapder)
      implicit real (a-h,o-z)
      dimension df(n,n,*),ud(n,n,*),fv(n,n,*),
     >  mapder(nq,*),mapcf(nq,nq,*)
      external as
c     Common block for equation parameters.
      parameter(mm=20,nqmx=2)
      common/eqn/mt,mx,my,neqn,peqn(100),
     >  acoef(0:mm,0:mm,0:mm,nqmx,nqmx,6),
     >  tmpt(0:mm),tmpx(0:mm),tmpy(0:mm)
      data istart /0/
      save istart

c     eqn_cfp evaluates F and DF for a nonlinear system of PDE's
c     of Caginalp-Fife phase field type ut = F(x,t,u,du,ddu):
c
c     ut = a1 \Delta u + a2 g(u) + a3 v
c
c     vt = b1 \Delta v - b2 v - b3 \Delta u - b4 g(u)  
c
c     where g(u) = 0.5 (u - u^3). Typical values for the parameters are
c
c     a1 = 1,	a2 = 1/e^2,	a3 = 2/e,
c
c     b1 = 1,	b2 = 1/2e,	b3 = 1/2,	b4 = 1/2e^2
c
c     where e is a small parameter proportional to the interface thickness.


      if(n.le.0.or.istart.le.0)then
         istart=1
c        Initialize template.

c        First equation (for ut).
c        ut = a1 \Delta u + a2 (u-u^3) + a3 v
c
c        First component (u): uxx,uxy,uyy,ux,uy,u.
         mapcf(1,1,1) =  1
         mapcf(1,1,2) = -1
         mapcf(1,1,3) =  1
         mapcf(1,1,4) = -1
         mapcf(1,1,5) = -1
         mapcf(1,1,6) =  1
c        Second component (v): vxx,vxy,vyy,vx,vy,v.
         mapcf(1,2,1) = -1
         mapcf(1,2,2) = -1
         mapcf(1,2,3) = -1
         mapcf(1,2,4) = -1
         mapcf(1,2,5) = -1
         mapcf(1,2,6) =  1

c        Second equation (for vt).
c        vt = b1 \Delta v - b2 v - b3 \Delta u - b4 g(u)  
c
c        First component (u): uxx,uxy,uyy,ux,uy,u.
         mapcf(2,1,1) =  1
         mapcf(2,1,2) = -1
         mapcf(2,1,3) =  1
         mapcf(2,1,4) = -1
         mapcf(2,1,5) = -1
         mapcf(2,1,6) =  1
c        Second component (v): vxx,vxy,vyy,vx,vy,v.
         mapcf(2,2,1) =  1
         mapcf(2,2,2) = -1
         mapcf(2,2,3) =  1
         mapcf(2,2,4) = -1
         mapcf(2,2,5) = -1
         mapcf(2,2,6) =  1

         return

      end if

      h=1.0/float(n)
c     System is phase field type: 
c     ut = a1 \Delta u + a2 (u-u^3) + a3 v
c     vt = b1 \Delta v - b2 v - b3 \Delta u - b4 g(u)  

c     Initialize coefficients.
      a1 = peqn(1)
      a2 = peqn(2)
      a3 = peqn(3)
      b1 = peqn(4)
      b2 = peqn(5)
      b3 = peqn(6)
      b4 = peqn(7)

c     Unit number for warning messages.
      io = 6
c     Check that all desired derivatives are present and set pointers.
      j11 = norerr(io,mapder(1,1))
      j13 = norerr(io,mapder(1,3))
      j16 = norerr(io,mapder(1,6))
      j21 = norerr(io,mapder(2,1))
      j23 = norerr(io,mapder(2,3))
      j26 = norerr(io,mapder(2,6))

c     Check that all desired coefficients are present and set pointers.
      k111 = norerr(io,mapcf(1,1,1))
      k113 = norerr(io,mapcf(1,1,3))
      k116 = norerr(io,mapcf(1,1,6))
      k126 = norerr(io,mapcf(1,2,6))
      k211 = norerr(io,mapcf(2,1,1))
      k213 = norerr(io,mapcf(2,1,3))
      k216 = norerr(io,mapcf(2,1,6))
      k221 = norerr(io,mapcf(2,2,1))
      k223 = norerr(io,mapcf(2,2,3))
      k226 = norerr(io,mapcf(2,2,6))

      do 100 i1=1,n
      do 100 i2=1,n
         x = i1*h
         y = i2*h
c        Extract derivatives from array ud.
         uxx = ud(i1,i2,j11)
         uyy = ud(i1,i2,j13)
         u   = ud(i1,i2,j16)
         vxx = ud(i1,i2,j21)
         vyy = ud(i1,i2,j23)
         v   = ud(i1,i2,j26)

c        Auxiliary functions.
         g0 = 0.50 * u * (1.0 - u**2)
         g1 = 0.5 - 1.50 * u**2

c        RHS of first equation.
c        ut = a1 \Delta u + a2 g(u) + a3 v
         fv(i1,i2,1) = a1 * (uxx + uyy) + a2 * g0 + a3*v

c        RHS of second equation.
c        vt = b1 \Delta v - b2 v - b3 \Delta u - b4 g(u)  
         fv(i1,i2,2) = b1 * (vxx+vyy) - b2 * v - b3 * (uxx+uyy) - b4 * g0 

c        Derivatives of first component of right-hand side.
c        Remember a*I+b*DF is what we are really evaluating!
         df(i1,i2,k111) =     b * a1
         df(i1,i2,k113) =     b * a1
         df(i1,i2,k116) = a + b * a2 * g1
         df(i1,i2,k126) =     b * a3

c        Derivatives of second component of right-hand side.
         df(i1,i2,k211) =     b * (-b3)
         df(i1,i2,k213) =     b * (-b3)
         df(i1,i2,k216) =     b * (-b4 * g1)

         df(i1,i2,k221) =     b * b1
         df(i1,i2,k223) =     b * b1
         df(i1,i2,k226) = a + b * (-b2)
100   continue

      return
      end
