      subroutine eqn_son(n,ud,a,b,t,as,f,df,nq,nd,mapcf,mapder)
      implicit real (a-h,o-z)
      dimension df(n,n,*),ud(n,n,*),f(n,n,*),
     >  mapder(nq,*),mapcf(nq,nq,*)
      external as
c     Common block for equation parameters.
      parameter(mm = 20,nqmx = 2)
      common/eqn/mt,mx,my,neqn,peqn(100),
     >  acoef(0:mm,0:mm,0:mm,nqmx,nqmx,6),
     >  tmpt(0:mm),tmpx(0:mm),tmpy(0:mm)
      data istart /0/
      save istart

c     eqn_son evaluates F and DF for a nonlinear system of PDE's
c     of Soner's phase field type u_t = F(x,t,u,du,ddu):
c
c     ut = a1 (uxx + uyy) - a2 w1(u) + a3 h0(u) v
c
c     vt = b1 (vxx + vyy) - b2 h0^2(u) v - b3 h0(u) (uxx + uyy) + b4 h0(u) w1(u)
c
c     where w0(u) = 0.5(1-u^2)^2, w1(u) = w0'(u), h0(u) = 1-u^2, and
c
c     a1 = 1,	a2 = 1/ee^2,	a3 = 1/ee,
c
c     b1 = 1,	b2 = 1/ee, 	b3 = 1,		b4 = 1/ee^2
c
c     where ee is a small parameter proportional to the interfacial 
c     thickness.


      if(n.le.0.or.istart.le.0)then
         istart = 1
c        Initialize template.

c        First equation (for ut).
c        ut = a1 (uxx + uyy) - a2 w1(u) + a3 h0(u) v
c
c        First component (u): uxx,uxy,uyy,ux,uy,u.
         mapcf(1,1,1) =  1
         mapcf(1,1,2) = -1
         mapcf(1,1,3) =  1
         mapcf(1,1,4) = -1
         mapcf(1,1,5) = -1
         mapcf(1,1,6) =  1
c        Second component (v): vxx,vxy,vyy,vx,vy,v.
         mapcf(1,2,1) = -1
         mapcf(1,2,2) = -1
         mapcf(1,2,3) = -1
         mapcf(1,2,4) = -1
         mapcf(1,2,5) = -1
         mapcf(1,2,6) =  1

c        Second equation (for v_t).
c        vt = b1 (vxx + vyy) - b2 h0^2(u) v - b3 h0(u) (uxx + uyy) 
c             + b4 h0(u) w1(u)
c
c        First component (u): uxx,uxy,uyy,ux,uy,u.
         mapcf(2,1,1) =  1
         mapcf(2,1,2) = -1
         mapcf(2,1,3) =  1
         mapcf(2,1,4) = -1
         mapcf(2,1,5) = -1
         mapcf(2,1,6) =  1
c        Second component (v): vxx,vxy,vyy,vx,vy,v.
         mapcf(2,2,1) =  1
         mapcf(2,2,2) = -1
         mapcf(2,2,3) =  1
         mapcf(2,2,4) = -1
         mapcf(2,2,5) = -1
         mapcf(2,2,6) =  1

         return

      end if

      h = 1.0/float(n)
c     System is phase field type: 

c     Initialize coefficients.
      ee = peqn(1)
      a1 = 1.0
      a2 = 1.0/ee**2
      a3 = 1.0/ee
      b1 = 1.0
      b2 = 1.0/ee
      b3 = 1.0
      b4 = 1.0/ee**2

c     Unit number for warning messages.
      io = 6
c     Check that all desired derivatives are present and set pointers.
      j11 = norerr(io,mapder(1,1))
      j13 = norerr(io,mapder(1,3))
      j16 = norerr(io,mapder(1,6))
      j21 = norerr(io,mapder(2,1))
      j23 = norerr(io,mapder(2,3))
      j26 = norerr(io,mapder(2,6))

c     Check that all desired coefficients are present and set pointers.
      k111 = norerr(io,mapcf(1,1,1))
      k113 = norerr(io,mapcf(1,1,3))
      k116 = norerr(io,mapcf(1,1,6))
      k126 = norerr(io,mapcf(1,2,6))
      k211 = norerr(io,mapcf(2,1,1))
      k213 = norerr(io,mapcf(2,1,3))
      k216 = norerr(io,mapcf(2,1,6))
      k221 = norerr(io,mapcf(2,2,1))
      k223 = norerr(io,mapcf(2,2,3))
      k226 = norerr(io,mapcf(2,2,6))

      do 100 i1 = 1,n
      do 100 i2 = 1,n
         x = i1*h
         y = i2*h
c        Extract derivatives from array ud.
         uxx = ud(i1,i2,j11)
         uyy = ud(i1,i2,j13)
         u   = ud(i1,i2,j16)
         vxx = ud(i1,i2,j21)
         vyy = ud(i1,i2,j23)
         v   = ud(i1,i2,j26)

c        Auxiliary functions.
         w0 = 0.50 * (1.0-u**2)**2
         w1 = -2.0 * u * (1.0 - u**2)
         w2 = -2.0 * (1.0-3.0*u**2)
         h0 = 1.0 - u**2
         h1 = -2.0 * u
c        RHS of first equation.
         f(i1,i2,1) = a1 * (uxx + uyy) - a2 * w1 + a3 * h0 * v

c        RHS of second equation.
         f(i1,i2,2) =  b1 * (vxx + vyy) - b2 * h0**2 * v 
     >               - b3 * h0 * (uxx + uyy) + b4 * h0 * w1

c        Derivatives of first component of right-hand side.
c        Irrelevant components have been omitted.
c        Remember a*I+b*DF is what we are really evaluating!
         df(i1,i2,k111) =     b * (a1)
         df(i1,i2,k113) =     b * (a1)
         df(i1,i2,k116) = a + b * (-a2 * w2 + a3 * h1 * v)
         df(i1,i2,k126) =     b * (a3 * h0)

c        Derivatives of second component of right-hand side.
         df(i1,i2,k211) =     b * (-b3 * h0)
         df(i1,i2,k213) =     b * (-b3 * h0)
         df(i1,i2,k216) =     b * (-b2 * 2.0 * h0 * h1 * v
     >                          - b3 * h1 * (uxx + uyy) 
     >                          + b4 * (h1 * w1 + h0 * w2))

         df(i1,i2,k221) =     b * (b1)
         df(i1,i2,k223) =     b * (b1)
         df(i1,i2,k226) = a + b * (-b2 * h0**2)

100   continue

      return
      end
