      subroutine eqn_ld(n,ud,a,b,t,as,fv,df,nq,nd,mapcf,mapder)
      implicit real (a-h,o-z)
      dimension df(n,n,*),ud(n,n,*),fv(n,n,*),
     >  mapder(nq,*),mapcf(nq,nq,*)
      external as
      data istart /0/
      save istart

c     eqn_ld evaluates F and DF for a diagonal linear system of PDE's
c     u_t = F(x,t,u,du,ddu)

      if(n.le.0.or.istart.le.0)then
         istart=1
c        Initialize template.
c        Loop over derivatives.
         do 10 id=1,nd
c           Set all template entries to false.
            do 20 iq=1,nq
               do 30 jq=1,nq
                  mapcf(iq,jq,id)=-1
30             continue
c              Set diagonal entries to true.
               mapcf(iq,iq,id)=1
20          continue
10       continue

         return
      end if

      h=1.0/float(n)
c     Calculate F and a*I+b*DF: linear case.

c     Loop over grid points.
      do 100 i1=1,n
      do 100 i2=1,n
c        Initialize coefficient marker; not derivative marker!!
         do 200 iq=1,nq
c           Working on iq'th equation:  zero f to start.
            fv(i1,i2,iq)=0.0
c           Sum over components jq=1,...,nq.
            do 300 jq=1,nq
c              Higher-order terms first.
               do 400 id=1,5
                  jd=mapcf(iq,jq,id)
                  if(jd.gt.0)then
c                    Equation depends on derivative number id.
                     c=as(t,i1*h,i2*h,iq,jq,id)
                     mder=mapder(jq,id)
                     fv(i1,i2,iq)=fv(i1,i2,iq)+c*ud(i1,i2,mder)
                     df(i1,i2,jd)=b*c
                  end if
400            continue
c              Zero-order terms.
               id=6
               jd=mapcf(iq,jq,id)
               if(jd.gt.0)then
                  c=as(t,i1*h,i2*h,iq,jq,id)
                  mder=mapder(jq,id)
                  fv(i1,i2,iq)=fv(i1,i2,iq)+c*ud(i1,i2,mder)
                  df(i1,i2,jd)=a+b*c
               end if
300         continue
200      continue
100   continue

      return
      end
      subroutine eqn_lq(n,ud,a,b,t,as,fv,df,nq,nd,mapcf,mapder)
      implicit real (a-h,o-y), complex(z)
      dimension df(n,n,*),ud(n,n,*),fv(n,*),
     >  mapder(nq,*),mapcf(nq,nq,*)
      external as
      data istart /0/
      save istart

      if(nq.ne.1)then
         write(6,*) ' nq=',nq,' not equal to 1; resetting nq=1'
         nq=1
      end if
      if(n.le.0.or.istart.le.0)then

         istart=1
c        Initialize template to all true.
         do 10 id=1,6
            mapcf(1,1,id)=1
10       continue
         return
      end if

      h=1.0/float(n)
c     Calculate F and a*I+b*DF: linear case.

      do 100 i1=1,n
      do 100 i2=1,n
c        Zero f to start.
         fv(i1,i2)=0.0
c        Higher-order terms.
         do 200 id=1,5
            jd=mapcf(1,1,id)
            if(jd.gt.0)then
c              Equation depends on derivative number id.
               coef=as(t,i1*h,i2*h,1,1,id)
               fv(i1,i2)=fv(i1,i2)+coef*ud(i1,i2,jd)
               df(i1,i2,jd)=b*coef
            end if
200      continue
c        Zero-order terms.
         id=6
         jd=mapcf(1,1,id)
         if(jd.gt.0)then
            coef=as(t,i1*h,i2*h,1,1,id)
            fv(i1,i2)=fv(i1,i2)+coef*ud(i1,i2,jd)
            df(i1,i2,jd)=a+b*coef
         end if
100   continue

      return
      end

      subroutine eqn_ls(n,ud,a,b,t,as,fv,df,nq,nd,mapcf,mapder)
      implicit real (a-h,o-y), complex(z)
      dimension df(n,n,*),ud(n,n,*),fv(n,n,*),
     >  mapder(nq,*),mapcf(nq,nq,*)
      external as
      data istart /0/
      save istart

c     eqn_ls evaluates F and DF for a general linear system of PDE's
c     u_t = F(x,t,u,du,ddu) with off-diagonal terms.

      if(n.le.0.or.istart.le.0)then
         istart=1
c        Initialize template.
c        Set second-order off-diagonal template entries false and
c        first and zeroth-order ones true. Set diagonal entries true
c        except for 2 and 5.
c        Loop over derivatives.
         do 10 id=1,nd
c           Loop over equations.
            do 20 iq=1,nq
c              Loop over components.
               do 30 jq=1,nq
                  if(id.le.4)mapcf(iq,jq,id)=-1
                  if(id.gt.4)mapcf(iq,jq,id)=1
30             continue
c              Set all diagonal entries to true.
               mapcf(iq,iq,id)=1
               if(id.eq.2)mapcf(iq,iq,id)=-1
               if(id.eq.5)mapcf(iq,iq,id)=-1
20          continue
10       continue

         return

      end if

      h=1.0/float(n)
c     Calculate F and a*I+b*DF: linear case.

c     Loop over grid points.
      do 100 i1=1,n
      do 100 i2=1,n
         do 200 iq=1,nq
c           Working on iq'th equation:  zero f to start.
            fv(i1,i2,iq)=0.0
c           Sum over components jq=1,...,nq.
            do 300 jq=1,nq
               do 400 id=1,6
                  jd=mapcf(iq,jq,id)
                  if(jd.gt.0)then
c                    Equation depends on derivative number id.
                     c=as(t,i1*h,i2*h,iq,jq,id)
                     mder=mapder(jq,id)
                     fv(i1,i2,iq)=fv(i1,i2,iq)+c*ud(i1,i2,mder)
                     df(i1,i2,jd)=b*c
                     if(iq.eq.jq.and.id.eq.6)
     >                  df(i1,i2,jd)=df(i1,i2,jd)+a
                  end if
400            continue
300         continue
200      continue
100   continue

      return
      end

      subroutine eqn_md(n,ud,a,b,t,as,fv,df,nq,nd,mapcf,mapder)
      implicit real (a-h,o-y), complex(z)
      dimension df(n,n,*),ud(n,n,*),fv(n,n,*),
     >  mapder(nq,*),mapcf(nq,nq,*)
      external as
      data istart /0/
      save istart

c     eqn_mcd evaluates F and DF for a diagonal nonlinear system of PDE's
c     of mean curvature type u_t = F(x,t,u,du,ddu).

      if(n.le.0.or.istart.le.0)then
         istart=1
c        Initialize template.
c        Set all off-diagonal template entries false.
c        Set all diagonal entries true.
c        Loop over derivatives.
         do 10 id=1,nd
c           Loop over equations.
            do 20 iq=1,nq
c              Loop over components.
               do 30 jq=1,nq
                  mapcf(iq,jq,id)=-1
30             continue
c              Set all diagonal entries to true.
               mapcf(iq,iq,id)=1
20          continue
10       continue

         return

      end if

      h=1.0/float(n)
c     Equation is minimal surface type: 
c     u_t = (1+u_y^2)u_xx + 2 u_x u_y u_xy + (1+u_x^2)u_yy 
c         + bx u_x + by u_y + c (u+u^3) + f(x,y)

      do 100 i=1,n
      do 100 j=1,n
         do 200 iq=1,nq
c           Zero rhs of iq'th equation.
            fv(i,j,iq)=0.0
c           Sum over components of iq'th equation.
            do 300 jq=iq,iq
c              Extract derivatives of jq'th component from arrays.
               j1=mapder(jq,1)
               j2=mapder(jq,2)
               j3=mapder(jq,3)
               j4=mapder(jq,4)
               j5=mapder(jq,5)
               j6=mapder(jq,6)
               uxx=ud(i,j,j1)
               uxy=ud(i,j,j2)
               uyy=ud(i,j,j3)
               ux =ud(i,j,j4)
               uy =ud(i,j,j5)
               u  =ud(i,j,j6)
c              Compute coefficients with as.
               x=i*h
               y=j*h
               bx =as(t,x,y,iq,jq,4)
               by =as(t,x,y,iq,jq,5)
               c  =as(t,x,y,iq,jq,6)

c              This needs some fixing for the non-diagonal case.
               fv(i,j,iq) =  fv(i,j,iq)+
     >               (1.0+uy**2)*uxx 
     >             + 2.0*ux*uy*uxy 
     >             + (1.0+ux**2)*uyy 
     >             + bx*ux 
     >             + by*uy 
     >             + c*(u+u**3)
               k1=mapcf(iq,jq,1)
               k2=mapcf(iq,jq,2)
               k3=mapcf(iq,jq,3)
               k4=mapcf(iq,jq,4)
               k5=mapcf(iq,jq,5)
               k6=mapcf(iq,jq,6)
               df(i,j,k1)=b*(1.0+uy**2)
               df(i,j,k2)=b*2.0*ux*uy
               df(i,j,k3)=b*(1.0+ux**2)
               df(i,j,k4)=b*( 2.0*uy*uxy + 2.0*ux*uyy + bx )
               df(i,j,k5)=b*( 2.0*ux*uxy + 2.0*uy*uxx + by )
               df(i,j,k6)=a+b*c*(1.0+3.0*u**2)

300         continue
200      continue
100   continue


      return
      end

      subroutine eqn_mq(n,ud,a,b,t,as,fv,df,nq,nd,mapcf,mapder)
      implicit real (a-h,o-y), complex(z)
      dimension df(n,n,*),ud(n,n,*),fv(n,n),
     >  mapder(nq,*),mapcf(nq,nq,*)
      external as
      data istart /0/
      save istart

      if(nq.ne.1)then
         write(6,*) ' nq=',nq,' not equal to 1; resetting nq=1'
         nq=1
      end if
      if(n.le.0.or.istart.le.0)then
         istart=1
c        Set template to all true.
         do 10 id=1,6
            mapcf(1,1,id)=1
10       continue
         return
      end if

      h=1.0/float(n)
c     Calculate F and a*I+b*DF: nonlinear case.

c     Equation is minimal surface type: 
c
c     u_t = (1+u_y^2)u_xx + 2 u_x u_y u_xy + (1+u_x^2)u_yy 
c         + bx u_x + by u_y + c (u+u^3) + f(x,y)
      do 100 i=1,n
      do 100 j=1,n
c        Extract u derivatives from arrays.
         uxx=ud(i,j,1)
         uxy=ud(i,j,2)
         uyy=ud(i,j,3)
         ux =ud(i,j,4)
         uy =ud(i,j,5)
         u  =ud(i,j,6)
c        Compute coefficients with as.
         x=i*h
         y=j*h
         bx =as(t,x,y,1,1,4)
         by =as(t,x,y,1,1,5)
         c  =as(t,x,y,1,1,6)

         fv(i,j) =   (1.0+uy**2)*uxx 
     >             + 2.0*ux*uy*uxy 
     >             + (1.0+ux**2)*uyy 
     >             + bx*ux 
     >             + by*uy 
     >             + c*(u+u**3)
         df(i,j,1)=b*(1.0+uy**2)
         df(i,j,2)=b*2.0*ux*uy
         df(i,j,3)=b*(1.0+ux**2)
         df(i,j,4)=b*( 2.0*uy*uxy + 2.0*ux*uyy + bx )
         df(i,j,5)=b*( 2.0*ux*uxy + 2.0*uy*uxx + by )
         df(i,j,6)=a+b*c*(1.0+3.0*u**2)

100   continue

      return
      end
      subroutine eqn_ms(n,ud,a,b,t,as,fv,df,nq,nd,mapcf,mapder)
      implicit real (a-h,o-y), complex(z)
      dimension df(n,n,*),ud(n,n,*),fv(n,n,*),
     >  mapder(nq,*),mapcf(nq,nq,*)
      external as
      data istart /0/
      save istart

c     eqn_mcd evaluates F and DF for a nonlinear system of PDE's
c     of mean curvature type u_t = F(x,t,u,du,ddu).

      if(n.le.0.or.istart.le.0)then
         istart=1
c        Initialize template.
c        Set second-order off-diagonal template entries false.
c        Set all diagonal entries and all first and zero-order entries true.
c        Loop over equations.
         do 10 iq=1,nq
c           Loop over components.
            do 20 jq=1,nq
               mapcf(iq,jq,1)=-1
               mapcf(iq,jq,2)=-1
               mapcf(iq,jq,3)=-1
               mapcf(iq,jq,4)= 1
               mapcf(iq,jq,5)= 1
               mapcf(iq,jq,6)= 1
20          continue
c           Set all diagonal entries to true.
            mapcf(iq,iq,1)= 1
            mapcf(iq,iq,2)= 1
            mapcf(iq,iq,3)= 1
            mapcf(iq,iq,4)= 1
            mapcf(iq,iq,5)= 1
            mapcf(iq,iq,6)= 1
10       continue

         return

      end if

      h=1.0/float(n)
c     Equation is minimal surface type: 
c     u_t = (1+u_y^2)u_xx + 2 u_x u_y u_xy + (1+u_x^2)u_yy 
c         + bx u_x + by u_y + c (u+u^3) + f(x,y)
c         = F(x,y,t,u,ux,uy,uxx,uxy,uyy)

      do 100 i=1,n
      do 100 j=1,n
         do 200 iq=1,nq
c           Zero rhs F of iq'th equation.
            fv(i,j,iq)=0.0
c           Sum over components of iq'th equation.
            do 300 jq=1,nq
c              Extract derivatives of jq'th component from arrays.
               j1=mapder(jq,1)
               j2=mapder(jq,2)
               j3=mapder(jq,3)
               j4=mapder(jq,4)
               j5=mapder(jq,5)
               j6=mapder(jq,6)
               uxx=0.0
               uxy=0.0
               uyy=0.0
               ux =0.0
               uy =0.0
               u  =0.0
               if(j1.gt.0)uxx=ud(i,j,j1)
               if(j2.gt.0)uxy=ud(i,j,j2)
               if(j3.gt.0)uyy=ud(i,j,j3)
               if(j4.gt.0)ux =ud(i,j,j4)
               if(j5.gt.0)uy =ud(i,j,j5)
               if(j6.gt.0)u  =ud(i,j,j6)
c              Compute first and zero-order coefficients with as.
               x=i*h
               y=j*h
               bx =as(t,x,y,iq,jq,4)
               by =as(t,x,y,iq,jq,5)
               c  =as(t,x,y,iq,jq,6)

c              Zero derivative entries.
               do 500 id=1,nd
                  jd=mapcf(iq,jq,id)
                  if(jd.gt.0)df(i,j,jd)=0.0
500            continue
                  
c              Add terms one at a time according to template.
c              Add contribution to each derivative simultaneously.
               k1=mapcf(iq,jq,1)
               if(k1.gt.0)then
                  fv(i,j,iq) =  fv(i,j,iq)+
     >               (1.0+uy**2)*uxx 
                  df(i,j,k1)=df(i,j,k1)+ b*(1.0+uy**2)
c                 Add contribution to uy component.
                  k5=mapcf(iq,jq,5)
                  if(k5.gt.0)then
                     df(i,j,k5)=df(i,j,k5)+ b* 2.0*uy*uxx
                  else 
                     write(6,*) ' error in eqn_mcs with k5=',k5
                  end if
               end if
 
               k2=mapcf(iq,jq,2)
               if(k2.gt.0)then
                  fv(i,j,iq) =  fv(i,j,iq)+
     >               2.0*ux*uy*uxy 
                  df(i,j,k2)=df(i,j,k2)+ b*2.0*ux*uy
c                 Add contribution to ux and uy components.
                  k4=mapcf(iq,jq,4)
                  if(k4.gt.0)then
                     df(i,j,k4)=df(i,j,k4)+ b* 2.0*uy*uxy
                  else 
                     write(6,*) ' error in eqn_mcs with k4=',k4
                  end if
                  k5=mapcf(iq,jq,5)
                  if(k5.gt.0)then
                     df(i,j,k5)=df(i,j,k5)+ b* 2.0*ux*uxy
                  else 
                     write(6,*) ' error in eqn_mcs with k5=',k5
                  end if
               end if

               k3=mapcf(iq,jq,3)
               if(k3.gt.0)then
                  fv(i,j,iq) =  fv(i,j,iq)+
     >               (1.0+ux**2)*uyy 
                  df(i,j,k3)=df(i,j,k3)+ b*(1.0+ux**2)
c                 Add contribution to ux component.
                  k4=mapcf(iq,jq,4)
                  if(k4.gt.0)then
                     df(i,j,k4)=df(i,j,k4)+ b* 2.0*ux*uyy
                  else 
                     write(6,*) ' error in eqn_mcs with k4=',k4
                  end if
               end if

               k4=mapcf(iq,jq,4)
               if(k4.gt.0)then
                  fv(i,j,iq) =  fv(i,j,iq)+
     >               bx*ux 
                  df(i,j,k4)=df(i,j,k4)+ b* bx
               end if

               k5=mapcf(iq,jq,5)
               if(k5.gt.0)then
                  fv(i,j,iq) =  fv(i,j,iq)+
     >               by*uy 
                  df(i,j,k5)=df(i,j,k5)+ b* by
               end if

               k6=mapcf(iq,jq,6)
               if(k6.gt.0)then
                  fv(i,j,iq) =  fv(i,j,iq)+
     >               c*(u+u**3)
                  if(iq.eq.jq)then
                     df(i,j,k6)=df(i,j,k6)+ a+b*c*(1.0+3.0*u**2)
                  else
                     df(i,j,k6)=df(i,j,k6)+ b*c*(1.0+3.0*u**2)
                  end if
               end if

300         continue
200      continue
100   continue


      return
      end
      subroutine eqn_zd(n,ud,a,b,t,as,fv,df,nq,nd,mapcf,mapder)
      implicit real (a-h,o-y), complex(z)
      dimension df(n,n,*),ud(n,n,*),fv(n,n,*),
     >  mapder(nq,*),mapcf(nq,nq,*)
      external as
      data istart /0/
      save istart

c     eqn_zd evaluates F and DF for a general linear system of PDE's
c     u_t = F(x,t,u,du,ddu) with off-diagonal terms.

      if(n.le.0.or.istart.le.0)then
         istart=1
c        Initialize template.
c        Set all off-diagonal template entries false.
c        Set diagonal entries true except for 5 (no uy term).
c        Loop over derivatives.
         do 10 id=1,nd
c           Loop over equations.
            do 20 iq=1,nq
c              Loop over components.
               do 30 jq=1,nq
                 mapcf(iq,jq,id)=-1
30             continue
c              Set all diagonal entries to true.
               mapcf(iq,iq,id)=1
c              Drop uy term.
               if(id.eq.5)mapcf(iq,iq,id)=-1
20          continue
10       continue
      write(6,*) '  iq   jq  mapcf'
      do 50 iq=1,nq
      do 50 jq=1,nq
         write(6,*) iq,jq,(mapcf(iq,jq,id),id=1,nd)
50    continue


         return

      end if

      h=1.0/float(n)
c     Calculate F and a*I+b*DF: linear case.

c     Loop over grid points.
      do 100 i1=1,n
      do 100 i2=1,n
         do 200 iq=1,nq
c           Working on iq'th equation:  zero f to start.
            fv(i1,i2,iq)=0.0
c           Sum over components jq=iq,...,iq since system is diagonal.
            do 300 jq=1,nq
               do 400 id=1,5
                  jd=mapcf(iq,jq,id)
                  if(jd.gt.0)then
c                    Equation depends on derivative number id.
                     mder=mapder(jq,id)
                     c=as(t,i1*h,i2*h,iq,jq,id)
                     fv(i1,i2,iq)=fv(i1,i2,iq)+c*ud(i1,i2,mder)
                     df(i1,i2,jd)=b*c
                  end if
400            continue
               id=6
               jd=mapcf(iq,jq,id)
               if(jd.gt.0)then
                  mder=mapder(jq,id)
                  c=as(t,i1*h,i2*h,iq,jq,id)
                  u=ud(i1,i2,mder)
                  fv(i1,i2,iq)=fv(i1,i2,iq)+c*(u+u**3)
                  df(i1,i2,jd)=a+b*c*(1.0+3.0*u**2)
               end if
300         continue
200      continue
100   continue

      return
      end

      subroutine eqn_zq(n,ud,a,b,t,as,fv,df,nq,nd,mapcf,mapder)
      implicit real (a-h,o-y), complex(z)
      dimension df(n,n,*),ud(n,n,*),fv(n,*),
     >  mapder(nq,*),mapcf(nq,nq,*)
      external as
      data istart /0/
      save istart

      if(nq.ne.1)then
         write(6,*) ' nq=',nq,' not equal to 1; resetting nq=1'
         nq=1
      end if
      if(n.le.0.or.istart.le.0)then
         istart=1
c        Set template as initialization.
         do 10 id=1,6
            mapcf(1,1,id)=1
10       continue
c        Drop uy term from equation.
         mapcf(1,1,5)=-1
         return
      end if

      h=1.0/float(n)
c     Calculate F and a*I+b*DF: nonlinear case.

      do 100 i1=1,n
      do 100 i2=1,n
c        Zero f to start.
         fv(i1,i2)=0.0
c        Higher-order terms.
         do 200 id=1,5
            jd=mapcf(1,1,id)
            if(jd.gt.0)then
               coef=as(t,i1*h,i2*h,1,1,id)
               fv(i1,i2)=fv(i1,i2)+coef*ud(i1,i2,jd)
               df(i1,i2,jd)=b*coef
            end if
200      continue
c        Zero-order terms: nonlinear.
         id=6
         jd=mapcf(1,1,id)
         if(jd.gt.0)then
            coef=as(t,i1*h,i2*h,1,1,id)

            u=ud(i1,i2,jd)
            fv(i1,i2)=fv(i1,i2)+coef*(u+u**3)
            df(i1,i2,jd)=a+b*coef*(1.0+3.0*u**2)
         end if

100   continue

      return
      end
      subroutine eqn_zs(n,ud,a,b,t,as,fv,df,nq,nd,mapcf,mapder)
      implicit real (a-h,o-y), complex(z)
      dimension df(n,n,*),ud(n,n,*),fv(n,n,*),
     >  mapder(nq,*),mapcf(nq,nq,*)
      external as
      data istart /0/
      save istart

c     eqn_zs evaluates F and DF for a general linear system of PDE's
c     u_t = F(x,t,u,du,ddu) with off-diagonal terms.

      if(n.le.0.or.istart.le.0)then
         istart=1
c        Initialize template.
c        Set second-order and uy off-diagonal template entries false
c        and ux and u ones true.
c        Set diagonal entries true except for 5 (no uy term).
c        Loop over derivatives.
         do 10 id=1,nd
c           Loop over equations.
            do 20 iq=1,nq
c              Loop over components.
               do 30 jq=1,nq
                 mapcf(iq,jq,id)=1
                 if(id.le.3)mapcf(iq,jq,id)=-1
                 if(id.eq.5)mapcf(iq,jq,id)=-1
30             continue
c              Set all diagonal entries to true.
               mapcf(iq,iq,id)=1
c              Drop uy term.
               if(id.eq.5)mapcf(iq,iq,id)=-1
20          continue
10       continue

         return

      end if

      h=1.0/float(n)
c     Calculate F and a*I+b*DF: linear case.

c     Loop over grid points.
      do 100 i1=1,n
      do 100 i2=1,n
         do 200 iq=1,nq
c           Working on iq'th equation:  zero f to start.
            fv(i1,i2,iq)=0.0
c           Sum over components jq=1,...,nq.
            do 300 jq=1,nq
               do 400 id=1,5
                  jd=mapcf(iq,jq,id)
                  if(jd.gt.0)then
c                    Equation depends on derivative number id.
                     mder=mapder(jq,id)
                     c=as(t,i1*h,i2*h,iq,jq,id)
                     fv(i1,i2,iq)=fv(i1,i2,iq)+c*ud(i1,i2,mder)
                     df(i1,i2,jd)=b*c
                  end if
400            continue
               id=6
               jd=mapcf(iq,jq,id)
               if(jd.gt.0)then
                  mder=mapder(jq,id)
                  c=as(t,i1*h,i2*h,iq,jq,id)
                  u=ud(i1,i2,mder)
                  fv(i1,i2,iq)=fv(i1,i2,iq)+c*(u+u**3)
c                 Add a to diagonal entries only.
                  if(iq.eq.jq)then
                     df(i1,i2,jd)=a+b*c*(1.0+3.0*u**2)
                  else
                     df(i1,i2,jd)=b*c*(1.0+3.0*u**2)
                  end if
               end if
300         continue
200      continue
100   continue

      return
      end
