      subroutine eqn_wbm(n,ud,a,b,t,as,f,df,nq,nd,mapcf,mapder)
      implicit real (a-h,o-z)
      dimension df(n,n, * ),ud(n,n, * ),f(n,n, * ),
     >  mapder(nq, * ),mapcf(nq,nq, * )
      external as
c     Common block for equation parameters.
      parameter(mm=20,nqmx=2)
      common/eqn/mt,mx,my,neqn,peqn(100),
     >  acoef(0:mm,0:mm,0:mm,nqmx,nqmx,6),
     >  tmpt(0:mm),tmpx(0:mm),tmpy(0:mm)
      data istart /0/
      save istart

c     eqn_wbm evaluates F and DF for a nonlinear system of PDE's
c     of WBM phase field type u_t = F(x,t,u,du,ddu):
c
c     u_t = am (e^2 \Delta u - fa v - fb (1-v) )
c
c     v_t = \Delta v 
c         + v(1-v) (fb-fa) \Delta u 
c         + (ux vx + uy vy) (1 - 2v) (fb-fa)
c         + (ux ux + uy uy) v(1-v) (fb' - fa')


      if(n.le.0.or.istart.le.0)then
         istart=1
c        Initialize template.

c        First equation (for ut).
c        ut = a (e^2 \Delta u - fa v - fb (1-v) )
c
c        First component (u): uxx,uxy,uyy,ux,uy,u.
         mapcf(1,1,1) =  1
         mapcf(1,1,2) = -1
         mapcf(1,1,3) =  1
         mapcf(1,1,4) = -1
         mapcf(1,1,5) = -1
         mapcf(1,1,6) =  1
c        Second component (v): vxx,vxy,vyy,vx,vy,v.
         mapcf(1,2,1) = -1
         mapcf(1,2,2) = -1
         mapcf(1,2,3) = -1
         mapcf(1,2,4) = -1
         mapcf(1,2,5) = -1
         mapcf(1,2,6) =  1

c        Second equation (for v_t).
c        vt = \Delta v 
c         + v(1-v) (fb-fa) \Delta u 
c         + (ux vx + uy vy) (1 - 2v) (fb-fa)
c         + (ux ux + uy uy) v(1-v) (fb' - fa')
c
c        First component (u): uxx,uxy,uyy,ux,uy,u.
         mapcf(2,1,1) =  1
         mapcf(2,1,2) = -1
         mapcf(2,1,3) =  1
         mapcf(2,1,4) =  1
         mapcf(2,1,5) =  1
         mapcf(2,1,6) =  1
c        Second component (v): vxx,vxy,vyy,vx,vy,v.
         mapcf(2,2,1) =  1
         mapcf(2,2,2) = -1
         mapcf(2,2,3) =  1
         mapcf(2,2,4) =  1
         mapcf(2,2,5) =  1
         mapcf(2,2,6) =  1

         return

      end if

      h=1.0/float(n)

c     Initialize coefficients.
      am = peqn(1)
      ee = peqn(2)
      wa = peqn(3)
      wb = peqn(4)
      aa = peqn(5)
      ab = peqn(6)

c     Unit number for warning messages.
      io=6
c     Check that all desired derivatives are present and set pointers.
      j11 = norerr(io,mapder(1,1))
      j13 = norerr(io,mapder(1,3))
      j14 = norerr(io,mapder(1,4))
      j15 = norerr(io,mapder(1,5))
      j16 = norerr(io,mapder(1,6))
      j21 = norerr(io,mapder(2,1))
      j23 = norerr(io,mapder(2,3))
      j24 = norerr(io,mapder(2,4))
      j25 = norerr(io,mapder(2,5))
      j26 = norerr(io,mapder(2,6))

c     Check that all desired coefficients are present and set pointers.
      k111 = norerr(io,mapcf(1,1,1))
      k113 = norerr(io,mapcf(1,1,3))
      k116 = norerr(io,mapcf(1,1,6))
      k126 = norerr(io,mapcf(1,2,6))
      k211 = norerr(io,mapcf(2,1,1))
      k213 = norerr(io,mapcf(2,1,3))
      k214 = norerr(io,mapcf(2,1,4))
      k215 = norerr(io,mapcf(2,1,5))
      k216 = norerr(io,mapcf(2,1,6))
      k221 = norerr(io,mapcf(2,2,1))
      k223 = norerr(io,mapcf(2,2,3))
      k224 = norerr(io,mapcf(2,2,4))
      k225 = norerr(io,mapcf(2,2,5))
      k226 = norerr(io,mapcf(2,2,6))

      do 100 i1=1,n
      do 100 i2=1,n
         x=i1 * h
         y=i2 * h
c        Extract derivatives from array ud.
         uxx = ud(i1,i2,j11)
         uyy = ud(i1,i2,j13)
         ux  = ud(i1,i2,j14)
         uy  = ud(i1,i2,j15)
         u   = ud(i1,i2,j16)
         vxx = ud(i1,i2,j21)
         vyy = ud(i1,i2,j23)
         vx  = ud(i1,i2,j24)
         vy  = ud(i1,i2,j25)
         v   = ud(i1,i2,j26)

c        Auxiliary functions.
         fa  = wa * u * (u-1) * (u-aa)
         fb  = wb * u * (u-1) * (u-ab)
         fm  = fb - fa
         fa1 = wa * ( u * (u-1) + u * (u-aa) + (u-1) * (u-aa) )
         fb1 = wb * ( u * (u-1) + u * (u-ab) + (u-1) * (u-ab) )
         fm1 = fb1 - fa1
         fa2 = 2 * wa * ( 3 * u -1 -aa )
         fb2 = 2 * wb * ( 3 * u -1 -ab )
         fm2 = fb2 - fa2
c        RHS of first equation.
         f(i1,i2,1) = am * ( ee**2 * (uxx + uyy) - fa * v - fb * (1-v) )

c        RHS of second equation.
         f(i1,i2,2) = vxx + vyy 
     >      + v * (1-v) * (fb-fa) * (uxx + uyy)
     >      + (1 - 2 * v) * (fb-fa) * (ux * vx + uy * vy)
     >      + v * (1-v) * (fb1-fa1) * (ux**2 + uy**2)

c        Derivatives of first component of right-hand side.
c        Remember a * I+b * DF is what we are really evaluating!
         df(i1,i2,k111) = b * am * ee**2
         df(i1,i2,k113) = b * am * ee**2
         df(i1,i2,k116) =  a + b * ( -am * v * fa1 - am * (1-v) * fb1 )
         df(i1,i2,k126) = b * ( am * fm )

         vmv=v * (1-v)
c        Derivatives of second component of right-hand side.
         df(i1,i2,k211) = b * ( vmv * fm )
         df(i1,i2,k213) = b * ( vmv * fm )
         df(i1,i2,k214) = b * ( (1 - 2 * v) * fm * vx + 2 * vmv * fm1 * ux )
         df(i1,i2,k215) = b * ( (1 - 2 * v) * fm * vy + 2 * vmv * fm1 * uy )
         df(i1,i2,k216) = b * ( vmv * fm1 * (uxx + uyy) 
     >           + (1 - 2 * v) * fm1 * (ux * vx + uy * vy)
     >           + vmv * fm2 * (ux**2 + uy**2) )

         df(i1,i2,k221) = b * ( 1 )
         df(i1,i2,k223) = b * ( 1 )
         df(i1,i2,k224) = b * ( (1 - 2 * v) * fm * ux )
         df(i1,i2,k225) = b * ( (1 - 2 * v) * fm * uy )
         df(i1,i2,k226) =  a + b * ( (1 - 2 * v) * fm * (uxx + uyy) 
     >           - 2 * fm * (ux * vx + uy * vy) 
     >           + (1 - 2 * v) * fm1 * (ux**2 + uy**2) )
100   continue

      return
      end
