c  ====================================================================
      subroutine evb(k,t,a,b,c)
c     .. Real/..
      implicit real (a-h,o-z)
c     .. Arrays ..
      dimension t(0:k),a(0:k,0:k),b(0:k),c(0:k)
c     .. 
c
c  Purpose
c  =======
c
c  This subroutine calculates the coefficients of the 
c  k-step variable-step BDF formula.
c  Given times t(i)=t_{n+1-i}, i=0(1)k, evb returns
c  two sets of coefficients b and c for the formulas
c
c     \sum_i=1^k b_i y[n+1,...,n+1-i] = h_n f_{n+1}
c
c  and 
c
c     \sum_i=0^k c_i y_{n+1-i} = h_n f_{n+1},
c
c  where h_n=t_{n+1}-t_n. The matrix a is used as workspace.
c  
c  Method
c  ======
c
c  The matrix a is constructed which implements the change of basis
c  from function values y_{n+1-i} to divided differences y[n+1,...,n+1-i]
c  for i=0(1)k. The coefficients b_i are used as workspace. Then the 
c  coefficients b_i are computed from the standard formulas in HNW I, 
c  p. 350, and the c coefficients are computed by applying the
c  transpose of the change of basis matrix.
c  
c  Arguments 
c  =========
c
c  k 	- integer.
c         On entry, contains the number of past steps t_{n+1-i}
c	  and coefficients b,c.  Unchanged on exit.
c
c  t 	- real array of dimension (0:k).
c         On entry, contains the last k+1 time locations t_{n+1-i}
c	  for i=0(1)k; t(i)=t_{n+1-i}. Unchanged on exit.
c
c  a 	- real array of dimension (0:k,0:k).
c         Used as workspace for change of basis matrix.
c
c  b 	- real array of dimension (0:k).
c         On exit, contains coefficients b_i for i=1(1)k.
c         b(0) is undefined but dimensioned for convenience.
c
c  c 	- real array of dimension (0:k).
c         On exit, contains coefficients c_i for i=0(1)k.
c
c  ====================================================================
c
c  .. Executable Statements ..
c
c  .. Compute change of basis matrix by differencing standard basis vectors ..
      do j=0,k
c  .. Compute column j of a ..
c  .. Fill b with jth standard basis vector ..
         do i=0,k
            b(i)=0.0
         end do
         b(j)=1.0
c  .. First row of a ..
         a(0,j)=b(0)
c  .. Index l is the order of the divided difference ..
         do l=1,k
c  .. Replace first k-l+1 elements of b with divided differences ..
            do i=0,k-l
               b(i)=(b(i)-b(i+1))/(t(i)-t(i+l))
            end do
c  .. This gives us an element of a ..
            a(l,j)=b(0)
         end do
      end do
c  .. Compute coefficients b for divided difference form of BDF ..
      b(1)=t(0)-t(1)
      do j=2,k
         b(j)=b(j-1)*(t(0)-t(j-1))
      end do
c  .. Multiply c=a^t b, noting that b(0) does not participate ..
      do i=0,k
         sum=0.0
         do j=1,k
            sum=sum+a(j,i)*b(j)
         end do
         c(i)=sum
      end do
      return
      end
