c  ====================================================================
      real function fs(x,y,t,ider,jq)
c     .. Real/..
      implicit real (a-h,o-z)
c     .. Common block ..
      common/sol/kt,kx,ky,nsol,psol(100)
c     .. 
c
c  Purpose
c  =======
c
c  Function fs returns exact values and derivatives 
c  of the jq'th component of a vector function according to
c  the following schedule, corresponding to the operator coeffs:
c  ider=0  f_t
c       1  f_xx
c       2  f_xy
c       3  f_yy
c       4  f_x
c       5  f_y
c       6  f 

c  The function is given by 
c
c  f(x,y,t)=pmn+(pmx-pmn)*(g(x-xsh,y-ysh,t)-gmn)/(gmx-gmn)
c
c  where 
c
c   g(x,y,t)=exp((a+b*cos(2pi*kt*t))*(a+b*cos(2pi*kx*x))*(a+b*cos(2pi*ky*y)),
c
c  kt,kx,ky,pmn,pmx,xsh and ysh (in psol) are stored in common block 'sol'
c  and set by the calling program.
c  The constants a and b in the formula for fs ensure that
c  one cosine vanishing does not freeze the other variables.
c
c  Other Routines Called 
c  =====================
c
c  none
c
c  ====================================================================
c
c  .. Executable Statements ..
c
      pmn=psol(4*(jq-1)+1)
      pmx=psol(4*(jq-1)+2)
      xsh=psol(4*(jq-1)+3)
      ysh=psol(4*(jq-1)+4)
      a=0.75
      b=0.25
      c=(a+b)**3
      d=(a+b)*(a-b)**2
      emx=exp(c)
      emn=exp(d)
      scale=(pmx-pmn)/(emx-emn)
      tpi=2.0*3.1415926535898
      st=tpi*kt
      sx=tpi*kx
      sy=tpi*ky
      xx=x-xsh
      yy=y-ysh
c     Three cosine factors go into the exponential.
      ct=a+b*cos(st*t)
      cx=a+b*cos(sx*xx)
      cy=a+b*cos(sy*yy)
c     The function itself.
      fs=exp(cx*cy*ct)
c     If only the function was required, we are done.
      if(ider.ge.6)then
c        Shift, scale and return.
         fs=pmn+scale*(fs-emn)
         return
      end if

c     Scale exponential: shift is destroyed by differentiation.
      fs=scale*fs

c     Inefficient but worry about it later.
c     First derivatives of cosine factors.
         ctp=-st*b*sin(st*t)
         cxp=-sx*b*sin(sx*xx)
         cyp=-sy*b*sin(sy*yy)

c     Second derivatives of cosine factors.
         ctpp=-st*st*b*cos(st*t)
         cxpp=-sx*sx*b*cos(sx*xx)
         cypp=-sy*sy*b*cos(sy*yy)

      if(ider.le.0)then
c        Evaluate f_t.
         fs=ctp*cx*cy*fs
      else if(ider.eq.1)then
c        Evaluate f_xx.
         fs=ct*(cxpp*cy+ct*cxp**2*cy**2)*fs
      else if(ider.eq.2)then
c        Evaluate f_xy.
         fs=ct*(cxp*cyp+ct*cx*cxp*cy*cyp)*fs
      else if(ider.eq.3)then
c        Evaluate f_yy.
         fs=ct*(cx*cypp+ct*cx**2*cyp**2)*fs
      else if(ider.eq.4)then
c        Evaluate f_x.
         fs=ct*cxp*cy*fs
      else if(ider.eq.5)then
c        Evaluate f_y.
         fs=ct*cx*cyp*fs
      end if

      return

      end
