      subroutine init_gsr(n,nq,u,rwk,nrw)
c     .. Real/..
      implicit real (a-h,o-z)
c     .. Arrays ..
      dimension u(n,n,nq),rwk(3,*)
c     .. Common block for solution parameters ..
      common/sol/kt,kx,ky,nsol,psol(100)
c     .. Statement function for periodic 1-d Gaussian with scale s ..
      th(x,s)=
     >  exp(-s*((x-5)**2))
     > +exp(-s*((x-4)**2))
     > +exp(-s*((x-3)**2))
     > +exp(-s*((x-2)**2))
     > +exp(-s*((x-1)**2))
     > +exp(-s*((x  )**2))
     > +exp(-s*((x+1)**2))
     > +exp(-s*((x+2)**2))
     > +exp(-s*((x+3)**2))
     > +exp(-s*((x+4)**2))
     > +exp(-s*((x+5)**2))


c     Initialize random number generator.
      idum=-1
c     Number of Gaussians.
      k=max(1,kx*ky)
c     Mesh size.
      h=1.0/float(n)
c     Set scale ss of Gaussians so that k Gaussians can fit in [0,1]^2
c     with no more than eps overlap. 
c     ss is 1/R^2 where R is the radius appearing in the Gaussian.
c     The radius where the Gaussian with scale length R is less than eps
c     is rho=sqrt(-log(eps))*R. To fit k of these circles in the box [0,1]^2,
c     we need k*pi*rho^2 = fraction (to leave 1-fraction of the box empty).
c     Thus R^2=rho^2/(-log(eps)) = 1/2*k*pi*(-log(eps)).

c     Original choices 96.06.08.
      eps=0.01
      fraction=0.8
c     Larger Gaussians 96.06.08.
      eps=0.05
      fraction=0.95

      pi=3.1415926535898
      rhosq=fraction/(k*pi)
      ss=-log(eps)/rhosq
      idum=-1
      ffggbb=ran1(idum)

c     Loop through components of solution.
      do 100 iq=1,nq

c        Unpack parameters from psol.
         pmn=psol(4*(iq-1)+1)
         pmx=psol(4*(iq-1)+2)
         xsh=psol(4*(iq-1)+3)
         ysh=psol(4*(iq-1)+4)

c        Generate k random centers and weights +-1 so Gaussians don't overlap
c        by more than eps.
         isign=-1
	 do 200 k1=1,k

c           Generate new center for nonoverlapping Gaussian and check it.
220         continue
            r1=ran1(idum) + xsh
            r2=ran1(idum) + ysh
c           Compute minimum distance to previous blobs.
            dminsq=1.0e+10*rhosq
            do 230 k2=1,k1-1
               d12=(r1-rwk(1,k2))**2+(r2-rwk(2,k2))**2
               dminsq=min(d12,dminsq)
230         continue
            if(dminsq.le.rhosq)goto 220

            isign=-isign
            rwk(1,k1)=r1
            rwk(2,k1)=r2
            rwk(3,k1)=isign
200      continue
210      continue

c        Evaluate sum of Gaussians on grid.
         do 300 i1=1,n
            x1=i1*h
            do 400 i2=1,n
               x2=i2*h
               sum=0.0
c              Sum over random sources.
               do 700 k1=1,k
                  x1k=rwk(1,k1)
                  x2k=rwk(2,k1)
                  wk =rwk(3,k1)
                  sum=sum+wk*th(x1-x1k,ss)*th(x2-x2k,ss)
700            continue
c              Store sum in u.
               u(i1,i2,iq)=sum
400         continue
300      continue

c        Shift and scale to make min equal to pmn and max equal to pmx.
         umx=arrmax(n*n,u(1,1,iq))
         umn=arrmin(n*n,u(1,1,iq))
         scale=(pmx-pmn)/(umx-umn)
         do 950 i2=1,n
         do 950 i1=1,n
            u(i1,i2,iq)=pmn+scale*(u(i1,i2,iq)-umn)
950      continue

100   continue
      return
      end

