c  ====================================================================
      subroutine nr12m(n,e,f,
     >         e1,e2,em,g1,g2,gm,f1,f2,fm,b1,b2,bm)
c     .. Real/..
      implicit real (a-h,o-z)
c     .. Array Arguments ..
      dimension f(n),e(n)
c     .. 
c
c  Purpose
c  =======
c
c  nr12m accepts two vectors e and f of length n, and returns the
c  1, 2 and max norms f1,f2 and fm of f, the 1,2 and max norms e1,
c  e2 and em of e, and the relative errors in 1, 2 and max norms
c  g1, g2 and gm.
c
c  Arguments 
c  =========
c
c  n 	- integer.
c         Size of arrays: should be at least 1. Unchanged on exit.
c
c  f 	- real array of dimension n.
c         Computed result; unchanged on exit.
c
c  e 	- real array of dimension n.
c         Exact result; unchanged on exit.
c
c  e1   - real.
c         On exit, absolute error in 1-norm: 
c            e1=\frac{1}{n} \sum_{i=1}^n |e(i)|.
c
c  e2   - real.
c         On exit, absolute error in 2-norm: 
c            e2=\sqrt{ \frac{1}{n} \sum_{i=1}^n |e(i)|^2 }.
c
c  em   - real.
c         On exit, absolute error in max-norm: 
c            em=\max_i |e(i)|.
c
c  g1   - real.
c         On exit, relative error in 1-norm: 
c            g1=(1/n)\sum_{i=1}^n |e(i)| / f1.
c
c  g2   - real.
c         On exit, relative error in 2-norm: 
c            g2=\sqrt{(1/n) \sum_{i=1}^n |e(i)|^2 } / f2.
c
c  gm   - real.
c         On exit, relative error in max-norm: 
c            gm=\max_i |e(i)| / fm.
c
c  f1   - real.
c         On exit, 1-norm of f : 
c            f1=\frac{1}{n} \sum_{i=1}^n |f(i)|.
c
c  f2   - real.
c         On exit, 2-norm of f : 
c            f2=\sqrt{ \frac{1}{n} \sum_{i=1}^n |f(i)|^2 }.
c
c  fm   - real.
c         On exit, max-norm of f : 
c            fm=\max_i |f(i)|.
c
c  b1   - real.
c         On exit, correct bits in 1-norm : 
c            b1= -log(g1)/log(2.0)
c
c  b2   - real.
c         On exit, correct bits in 2-norm : 
c            b2= -log(g2)/log(2.0)
c
c  bm   - real.
c         On exit, correct bits in max-norm : 
c            bm= -log(gm)/log(2.0)
c
c  Other Routines Called 
c  =====================
c
c  saxpy,sasum,snrm2,isamax
c
c  ====================================================================
c
c  .. Executable Statements ..
c
      f1=sasum(n,f,1)/float(n)
      e1=sasum(n,e,1)/float(n)
      f2=snrm2(n,f,1)/sqrt(float(n))
      e2=snrm2(n,e,1)/sqrt(float(n))
      fm=abs(f(isamax(n,f,1)))
      em=abs(e(isamax(n,e,1)))
      gm=1.0
      g1=1.0
      g2=1.0
      if(fm.gt.0.0)gm=em/fm
      if(f1.gt.0.0)g1=e1/f1
      if(f2.gt.0.0)g2=e2/f2
      b1=0.0
      b2=0.0
      bm=0.0
      if(g1.gt.0.0)b1=-log(g1)/log(2.0)
      if(g2.gt.0.0)b2=-log(g2)/log(2.0)
      if(gm.gt.0.0)bm=-log(gm)/log(2.0)
      return
      end
