c  ====================================================================
      subroutine optstep(dt,rhs,top,bot,k,dtopt)
c     .. Real..
      implicit real (a-h,o-z)
c     .. Arrays ..
      dimension top(0:k),bot(0:k)
c     .. 
c
c  Purpose
c  =======
c
c  This routine computes the solution dtopt of the time step equation
c
c     \frac{dtopt}{dt} 
c          \prod_{i=0}^{k-1} \frac{dtopt+top(0)-top(i)}{dt+bot(0)-bot(i)} = rhs
c
c  crudely to one or two-digit relative accuracy. It should be called from
c  inside the Newton iteration with top=bot=t(0) and from the estimation of
c  the next time step and order with top=t(0), bot=t(1).
c  ====================================================================
c
c  .. Executable Statements ..
c
      dtopt=dt
      rat=dtopt/dt
      do i=1,k
         rat=rat*(dtopt+top(0)-top(i-1))/(dt+bot(0)-bot(i-1))
      end do
ccc      write(6,*) ' Initial rat,rhs= ',rat,rhs
      
      if(rat.eq.rhs)return

ccc      ndec=0
ccc      ninc=0
ccc      nbis=0

      if(rat.gt.rhs)then
c        Halve time step and re-evaluate.
         fac=1.0/2.0
1        continue
ccc            write(6,*) ' Decreasing rat= ',rat
            dtopt=fac*dtopt
            rat=dtopt/dt
            do i=1,k
               rat=rat*(dtopt+top(0)-top(i-1))/(dt+bot(0)-bot(i-1))
            end do
ccc            ndec=ndec+1
            if(rat.le.rhs)goto 3
         goto 1
      end if

      if(rat.lt.rhs)then
c        Double time step and re-evaluate.
         fac=2.0
2        continue
ccc            write(6,*) ' Increasing rat= ',rat
            dtopt=fac*dtopt
            rat=dtopt/dt
            do i=1,k
               rat=rat*(dtopt+top(0)-top(i-1))/(dt+bot(0)-bot(i-1))
            end do
ccc            ninc=ninc+1
            if(rat.ge.rhs)then
               dtopt=dtopt/fac
               goto 3
            end if
         goto 2
      end if

3     continue
c     Now optimal step lies between dtopt and 2*dtopt.
c     Zero in with bisection.

c     Now r1 should be <rhs.
      d1=dtopt
      r1=d1/dt
      do i=1,k
         r1=r1*(d1+top(0)-top(i-1))/(dt+bot(0)-bot(i-1))
      end do

c     Now r2 should be >rhs.
      d2=2.0*dtopt
      r2=d2/dt
      do i=1,k
         r2=r2*(d2+top(0)-top(i-1))/(dt+bot(0)-bot(i-1))
      end do

      if(r1.gt.rhs.or.r2.lt.rhs)then
         write(6,*) ' dtopt not bracketed' 
         stop
      end if

      dd=d2-d1
      dacc=0.001*(d1+d2)
      do while(dd.gt.dacc)
         dd=0.50*dd
         dm=d1+dd
         rm=dm/dt
         do i=1,k
            rm=rm*(dm+top(0)-top(i-1))/(dt+bot(0)-bot(i-1))
         end do
ccc         nbis=nbis+1
         if(rm.le.rhs)d1=dm
      end do

      dtopt=dm
ccc      write(6,*) ' ndec,ninc,nbis,dt,dtopt=',ndec,ninc,nbis,dt,dtopt

      return
      end
