c  ====================================================================
      subroutine plterr(ms,ns,ne,times,errs,io,iofil,jq,date)
c     .. Real/..
      implicit real (a-h,o-z)
c     .. Arrays ..
      dimension times(0:ms),errs(0:ms,ne),io(*)
c     .. Character arrays ..
      character iofil*12,date*25,
     >nrm(9)*1,xcap*50,ycap*50,ncom(9)*1
c     .. Logical variables ..
      logical lbox,lcap
c     .. Captions for error/time graphs ..
      data ycap/'     Correct Lx-norm bits in uz       '/
c     .. Add nrm in position 15 of ycap ..
      data nrmpos/15/
c     .. Add jq in position 31 of ycap ..
      data jqpos/31/

c     .. Norm identifiers ..
      data (nrm(i),i=1,9)/'1','2','m','a','b','c','d','e','f'/
c     .. Component identifiers ..
      data (ncom(i),i=1,9)/'1','2','3','4','5','6','7','8','9'/
c     .. 
c
c  Purpose
c  =======
c
c  Routine plterr plots the correct bits (usually log_2 of the errors
c  in three different norms) stored in array errs on a PostScript file.
c  
c  Arguments 
c  =========
c
c  ms 	- integer.
c         First dimension of array errs: unchanged on exit.
c
c  ns 	- integer.
c         Number of time steps stored in array errs: unchanged on exit.
c
c  ne 	- integer.
c         Number of different norms: usually equal to 3: unchanged on exit.
c
c  times- real array of dimension (0:ms)
c	  Times to be plotted. Unchanged on exit.
c
c  errs - real array of dimension (0:ms,ne)
c	  Errors to be plotted, for j=0(1)ns. Components are errors in
c         various norms.  Unchanged on exit.
c
c  io   - integer array of input dimension.
c         Input parameters for plot controls.
c         iunit=io( 1) Initial output unit: uses iu=iunit through iunit+ne-1.
c         iq   =io( 2) Control of superimposition; iq>0 means put each plot
c		in its own file, iq=0 plot nothing at all, iq<0 means 
c		superimpose plots as controlled by inc.
c         ibeg =io( 3) Beginning of file number in iofil if iq>0.
c         iend =io( 4) End of file number in iofil if iq>0.
c         ifil =io( 5) File number to be put in iofil if inc>0 and iq>0.
c         inc  =io( 6) Control of file advancing: if iq>0, each plot is
c		put in a file with ifil larger by inc than the previous file.
c		If iq<0, inc>0 means start a new plot file, inc=0 means 
c		continue plotting in the file already open, and inc<0 means
c		terminate the plot file.
c         ipos =io(11) Position of norm indicator in iofil.
c         ipsz =io(12) Point size for captions and labels.
c         bmin =io(13) Minimum of correct bits plotted.
c         bmax =io(14) Maximum of correct bits plotted.
c         nxt  =io(15) Number of x tick marks.
c         nyt  =io(16) Number of y tick marks.
c         icap =io(17) icap>0 means write file name and date in x-caption,
c		icap<=0 means write 'Time' in x-caption.
c  iofil - character variable of length 12.
c	  Contains basename of output file.
c jq     - integer.
c         Specifies component being plotted--goes into y-caption
c  date  - character variable of length 25.
c         Gives date and time of run; set in calling program; -->y-caption.
c
c  Other Routines Called 
c  =====================
c
c  mknumfile
c  drawinit
c  drawgraphinit
c  drawgraph
c  drawgraphterm
c  drawterm
c
c  ====================================================================
c
c  .. Executable Statements ..
c
c     .. Unpack parameters from io ..
      iunit=io( 1)
      iq   =io( 2)
      ibeg =io( 3)
      iend =io( 4)
      ifil =io( 5)
      inc  =io( 6)
      ipos =io(11)
      ipsz =io(12)
      bmin =io(13)
      bmax =io(14)
      nxt  =io(15)
      nyt  =io(16)
      icap =io(17)

c     .. Return if iq=0 ..
      if(iq.eq.0)return

c     .. Hardwired plot parameters ..
c     .. Dimensionless length of x-ticks ..
      xt=0.02
c     .. Dimensionless length of y-ticks ..
      yt=0.02
c     .. Dimensionless line thickness ..
      th=0.002
c     .. Dimensionless dot radius ..
      dr=0.002
c     .. Curve gray (0=black, 1=white) ..
      cg=0.0
c     .. Dot gray (0=black, 1=white) ..
      dg=0.0
c     .. No dash pattern ..
      ndash=0
c     .. Left edge of time plot ..
      t0=times(0)
c     .. Right edge of time plot ..
      t2=times(ns)
      
c     .. Loop through L1, L2 and Lm errors.
      do ie=1,ne
      iu=iunit+ie-1

      if(iq.gt.0.or.inc.gt.0)then
c        .. Initialize plot file ..
c        .. Put ifil in iofil variable in positions ibeg through iend ..
         call mknumfile(iofil,ibeg,iend,ifil)
c        .. Put nrm(ie) in iofil variable in position ipos ..
         iofil(ipos:ipos)=nrm(ie)
c        .. Open iofil on unit iu; 'eof' argument means open at end 
c        .. of file if it already exists ..
         open(unit=iu,form='formatted',fileopt='eof',
     >   file=iofil,status='unknown',access='sequential')
c        .. Update io with new file number ..
         io(5)=ifil+inc
c        .. Initialize postscript graphics ..
         lbox=.false.
         lcap=.false.
         call drawinit(iu,lbox,lcap)
c        .. Label y-caption with type of norm ..
         ycap(nrmpos:nrmpos)=nrm(ie)
c        .. Label y-caption with component of solution ..
         ycap(jqpos:jqpos)=ncom(jq)
c        .. Label x-caption with time ..
         xcap='               Time                   '
         if(icap.gt.0)then
c           .. Write filename and date to x-caption ..
            write(xcap,9100) iofil,date
9100        format(4x,a12,2x,a25)
         end if
         call drawgraphinit(iu,t0,t2,t0,bmin,bmax,bmin,
     >    th,cg,nxt,nyt,xt,yt,xcap,ycap,ipsz)
      end if

c     .. Truncate correct bits with bmin and bmax ..
      do i=0,ns
         errs(i,ie)=max(bmin,min(bmax,errs(i,ie)))
      end do
c     .. Plot ..
      call drawgraph(iu,ns+1,times(0),errs(0,ie),
     >t0,t2,bmin,bmax,dr,dg,ndash,dash,th,cg)

      if(iq.gt.0.or.inc.lt.0)then
c        .. Terminate plots and PostScript graphics ..
         call drawgraphterm(iu)
         call drawterm(iu)
c        .. Close file on unit iu ..
         close(iu)
      end if

c     .. End of loop over different norms ..
      end do

      return
      end

