c  ====================================================================
      subroutine sstart(i,idum,n,nq,s,f)
c     .. Real/..
      implicit real (a-h,o-z)
c     .. Arrays ..
      dimension s(*),f(*)
c     .. 
c
c  Purpose
c  =======
c
c  Routine sstart sets the initial guess s for GMRES.
c  
c  Arguments 
c  =========
c  
c  i  - integer.
c	Flag for selecting start value:
c		s = random if i <1 (usually a terrible way to start).
c		s = 0 if i =1.
c		s = rhs f of eqn to be solved if i=2.
c		s = s+f if i=3.
c
c  idum  - integer.
c	Seed for random number generator. idum=-1 starts new sequence.
c
c  n  - integer.
c	Number of grid points per side of square grid on [-1,1]^2.
c
c nq -  integer.
c	Number of components of f and s.
c
c  s  - real array of size at least n*n*nq.
c	On exit, s contains start values for GMRES.
c
c  f  - real array of size at least n*n*nq.
c	On entry, f contains the right-hand side of the equation As=f 
c	to be solved by GMRES. Unchanged on exit.
c
c  ====================================================================
c
c  .. Executable Statements ..
c
c  .. Size of arrays s and f ..
      nnq=n*n*nq

      if(i.lt.1)then
c     .. Random values in [-1,1] ..
         call arrran1(idum,nnq,-1.0,1.0,s)
      else if(i.eq.1)then
c     .. Zero out array s ..
         do j=1,nnq
            s(j)=0.0
         end do
      else if(i.eq.2)then
c     .. Set s equal to right-hand side f of equation ..
         do j=1,nnq
            s(j)=f(j)
         end do
      else if(i.eq.3)then
c     .. Set s equal to the result of one step of forward Euler ..
         do j=1,nnq
            s(j)=s(j)+f(j)
         end do
      end if

      return
      end
