#! /bin/csh -f
#csh command for making ps files into latex pages,
# (nrows * ncols)  at a time.

#Query user for parameters.
#
echo '% Enter output filename (e.g. out.tex) ' 
set outfile = $<
echo "% outfile= $outfile"  >$outfile
echo "% outfile= $outfile" 
#
#Prefix of input files.
#Assume input files have the form [inprefix].123.ps.
echo '% Enter input filename prefix (e.g. abc in abc.123.ps) ' 
set inprefix = $<
echo "% filename= $inprefix"  >$outfile
echo "% filename= $inprefix" 
#
#First input file number.
echo '% Enter first file number ' 
set ifile = $<
echo "% ifile= $ifile"  >>$outfile
echo "% ifile= $ifile"  
#
#Number of input files.
echo '% Enter number of input files' 
set nfile = $<
echo "% nfile= $nfile"  >>$outfile
echo "% nfile= $nfile"  
#Stop at file number iend.
@ ifile = $ifile - 1
@ iend = $ifile + $nfile
#
#Number of rows per page
echo '% Enter number of rows (e.g. 2 or 3) ' 
set nrows = $<
echo "% nrows= $nrows"  >>$outfile
echo "% nrows= $nrows"  
#
#Number of columns per page
echo '% Enter number of columns (e.g. 2 or 3) ' 
set ncols = $<
echo "% ncols= $ncols"  >>$outfile
echo "% ncols= $ncols"  

#Set number of pages.
@ npages = $nfile / ( $nrows * $ncols )
echo "% npages= $npages"  >>$outfile
echo "% npages= $npages"  
#
#Query user for picture height, width, spacing and offset.
echo '% Enter height of each figure (e.g. 2x2: 4.0in, 3x3: 2.67in) ' 
set height = $<
echo "% height= $height"  >>$outfile
echo "% height= $height"  
echo '% Enter width of each figure (e.g. 2x2: 3.0in, 3x3: 2.0in) ' 
set width = $<
echo "% width= $width"  >>$outfile
echo "% width= $width"  
echo '% Enter vertical spacing between figures (e.g. 2x2: 0.5in 3x3: 0.3in) ' 
set spacing = $<
echo "% spacing= $spacing"  >>$outfile
echo "% spacing= $spacing"  
echo '% Enter offset at top of page (e.g. -1.0in) ' 
set offset = $<
echo "% offset= $offset"  >>$outfile
echo "% offset= $offset"  

#Current page number counter
set ipage = 0

echo ' \input{psfig} '  >>$outfile
echo ' \psfull '  >>$outfile
echo ' \documentstyle{article} '  >>$outfile
echo ' \pagestyle{myheadings} '  >>$outfile
echo ' \markright{'$outfile `date`'}'  >>$outfile
echo ' \begin{document} '  >>$outfile

while ( $ipage < $npages)
   @ ipage = $ipage + 1
   echo "% Processing page $ipage"  >>$outfile
   echo ' \begin{figure}[p] '  >>$outfile
   echo ' \vspace{'$offset'}'   >>$outfile

   set irows = 0
   while ( $irows < $nrows)
      @ irows = $irows + 1
      echo '\vspace{'$spacing'}'   >>$outfile
      echo '\centerline{'   >>$outfile

      set icols = 0
      while ( $icols < $ncols)
         @ icols = $icols + 1
         @ ifile = $ifile + 1
	 if( $ifile > $iend)then
	    break;break;break
	 else
            echo "\psfig{file=$inprefix.$ifile.ps,height=$height,width=$width}"   >>$outfile
	 endif
      end

      echo '}'   >>$outfile
   end

   echo '\end{figure}'  >>$outfile
   echo '\clearpage'  >>$outfile

end

echo '\end{document}'   >>$outfile
